/*
*  $Log: choose.c,v $
*  Revision 1.5  2004/01/31 14:48:48  Default
*  bug fix. Anyone could pick up the Ice Crown!
*
*  Revision 1.4  2004/01/30 22:32:20  Default
*  fix disappearing troops bug
*
*  Revision 1.3  2004/01/27 20:19:13  richq
*  The temp lord object has changed - it's now a pointer to the actual
*  table entry and gets rid of the need for all the copying
*  from temp to the table and back again.
*
*  Revision 1.2  2004/01/25 21:08:30  richq
*  The Choose screen - the one originally called with "T".
*
*  Revision 1.1  2004/01/22 22:22:56  richq
*  The screen shown when you want to seek, hide, fight, etc.
*
*
*/

#include "choose.h"
#include "think.h"
#include "select.h"
#include "lords.h"
#include "objects.h"
#include "tokens.h"
#include "tokendata.h"
#include "savearea.h"
#include "army.h"
#include "features.h"
#include "menu.h"
#include "gfx.h"
#include "icefear.h"
#include "doomdark.h"
#include "look.h"

// which choice is selected
int g_hiliteChoice;

void seek(void);
void hideLord(void);
void startFight(void);
void recruitLord(void);
void recruitMen(void);
void guardMen(void);
void battle(void);

// jump table
void (*const choiceTable[]) (void) = {
  seek,
  hideLord,
  startFight,
  recruitLord,
  recruitMen,
  guardMen,
  battle
};

void addChoice(char indicator) {
  addNewlineToken();
  addLiteralToken('#');
  addLiteralToken(indicator);
  addUppercaseToken();
  g_selectableThingTable[g_selectableThings++] = (u8)indicator;
}


// fdfa
void wannaFight() {
  if (g_locdat.object > OBJECT_NOTHING && g_locdat.object < OBJECT_WILD_HORSES) {
    addChoice('3');
    addToTokenList(fightThe);
    addToTokenList(objectDescriptions[g_locdat.object]);
  }
}

void newRecruits() {
  
  int lordHere = getRecruitableID();
  if (lordHere != 0) {
    setSelectedLord(lordHere);
    addChoice('4');
    addSingleToken(0xf6);   // recruit
    createLordTitle(g_selectedLord->title, g_selectedLord->firstname);
  }
  setSelectedLord(g_saveArea.selectedLord);
  
}


void citadelKeepChoices() {
  if (g_locdat.terrain == KEEP ||g_locdat.terrain == CITADEL) {
    
    if (g_garrisons[g_tmpGarrisonID].race != g_selectedLord->race)
      return;
    
    if (g_garrisons[g_tmpGarrisonID].size >= 0x19) {
      // more than 125 here, can recruit...
      int mySoldiers;
      if (g_garrisons[g_tmpGarrisonID].type)
        mySoldiers = g_selectedLord->riders;
      else
        mySoldiers = g_selectedLord->warriors;
      
      if (mySoldiers < 235) {
        addChoice('5');
        addToTokenList(recruitmen);
      }
    }
    if ( g_garrisons[g_tmpGarrisonID].size < 0xeb && // fix disappearing troops bug 
         ((g_garrisons[g_tmpGarrisonID].type == 0 && g_selectedLord->warriors >= 20) ||
         (g_garrisons[g_tmpGarrisonID].type  && g_selectedLord->riders >= 20))
    ) {
      addChoice('6');
      addToTokenList(standOnGuard);
    }
  }
}

void battleChoice(void) {
  if (g_selectedLord->courage && g_doomindex == 0) {
    // we are brave and not already in a fight
    print_tokens_pal(2);
    workOutLocationDetails(&g_inFront);
    if (g_doomindex == 0)
      return;
    
    if (g_freeindex >= 0x1d)
      return;
    
    addChoice('7');
    addToTokenList(tobattle);
  }
  
}


// use g_selectableThings and g_selectableThingTable[32] to
// keep track of what we can do
void createChoices() {
  int i;
  g_selectableThings = 0;
  for (i = 0; i < 7; i++) {
    g_selectableThingTable[i] = 0xff;
  }
  
  if (!g_selectedLord->hidden) {
    // #1 Seek
    addChoice('1');
    addSingleToken(0xf0);  
  }
  // now check if we can hide...
  if ( !(g_selectedLord->warriors || g_selectedLord->riders || 
    g_saveArea.selectedLord == MORKIN)) 
  {
    // we may hide
    addChoice('2');
    // what if we are already hidden?
    if (g_selectedLord->hidden) {
      // we can only unhide ourselves
      addToTokenList(donothide);
      return;
    }
    addSingleToken(0xee);  //hide
    if (g_doomindex) {
      // doomdark forces are here. 
      // That limits our chances of doing anything else...
      return;
    }
  }
  // got here? check we aren't in a fight!
  // fdfa
  wannaFight();
  // what about allies?
  newRecruits();
  citadelKeepChoices();
  battleChoice();
  
  
}

//6396
void choose(void) {
  updateCurrentInfo();
  calculateCourage();
  setObjectHere();
  // if dead, show think
  if (!g_selectedLord->alive) {
    think();
    return;
  }
  // fd76 clear display, do initi text
  clearAndPressKeyText();
  
  createChoices();
  addNewlineToken();
  print_tokens_pal(2);
  
  updateCurrentInfo();
  g_lordScreen = CHOOSE_SCREEN;
  
  // highlight the first object
  g_hiliteChoice = 0;
  selectOption(g_hiliteChoice);
  
}




void makeChoice(void) {
  int choice = g_selectableThingTable[g_hiliteChoice]-'1';
  
  choiceTable[choice]();
  
}

void chooseDown(void){
  g_hiliteChoice++;
  if (g_hiliteChoice < g_selectableThings) {
    selectOption(g_hiliteChoice);
    
  } else {
    g_hiliteChoice--;
  }
  
}

void chooseUp(void){
  if (g_hiliteChoice) {
    g_hiliteChoice--;
    selectOption(g_hiliteChoice);
  }
}



void seek(void){
  clearHighlighter();
  
  g_tmpObject = 0;
  g_objectKey = 0;
  
  if (g_locdat.object >= OBJECT_WILD_HORSES) {
    g_objectKey = g_locdat.object;
    g_tmpObject = g_locdat.object;
    if (g_locdat.object == OBJECT_WILD_HORSES) {
      if (g_selectedLord->race < 6) {
        g_selectedLord->horse = 1;
        
        calculateGFX();  
      }
    } else if (g_locdat.object > OBJECT_WILD_HORSES) {
      switch (g_locdat.object) {
        case OBJECT_GUIDANCE :
          g_objectKey = randomish()&0x1f;
          g_locdat.object = 0;
          break;
        case OBJECT_SHELTER :
          updateStats(16);
          g_tmpObject = g_locdat.object;
          g_locdat.object = 0;
          break;
        case OBJECT_HAND :
          g_selectedLord->time = 0;
          g_locdat.object = 0;
          break;
        case OBJECT_CUP:
          g_selectedLord->time = 16;
          g_locdat.object = 0;
          break;
        case OBJECT_WATERS :
          g_selectedLord->energy = 0x78;
          g_selectedLord->riders_energy = 0x78;
          g_selectedLord->warriors_energy = 0x78;
          g_locdat.object = 0;
          break;
        case OBJECT_SHADOWS :
          g_selectedLord->energy = 0;
          g_selectedLord->riders_energy = 0;
          g_selectedLord->warriors_energy = 0;
          g_locdat.object = 0;
          break;
        default : // something we pick up...
          if ( (g_locdat.object >= OBJECT_ICE_CROWN &&  g_saveArea.selectedLord == MORKIN) ||
                (g_locdat.object < OBJECT_ICE_CROWN && g_selectedLord->object < OBJECT_ICE_CROWN)
          ) {
            // only morkin can pick up the moonring/ice crown
            // if the lord has the moonring/ice crown, do not drop it to pick a sword up!
            g_tmpObject = g_locdat.object;
            g_locdat.object = g_selectedLord->object;
            g_selectedLord->object = g_tmpObject;
          } else {
            g_tmpObject = 0;
            g_objectKey = 0;
          }
          break;
      }
      saveLocationData();
    }
  }
  thinkWithObjectSet();
}

void hideLord(void){
  g_selectedLord->hidden ^= 1;
  clearHighlighter();
  think();
  
}

void startFight(void){
  // 6492
  clearHighlighter();

  if (!g_freeindex && (g_selectedLord->object - 0xb) != g_locdat.object) {
    // no armies and we don't have the sword Objectslayer
    fightEnemy();
    if (!g_selectedLord->alive) {
      // oh man. we lost!
      g_selectedLord->deathby = g_locdat.object;
      think();
      return;
    }
  } 
  // else we won the fight
  
  g_tmpObject = g_locdat.object;
  g_objectKey = g_locdat.object;
  g_locdat.object = 0;
  
  saveLocationData();
  thinkWithObjectSet();
}

void recruitLord(void){
  
  int lordHere = getRecruitableID();
  
  g_saveArea.selectedLord = lordHere;
  
  updateCurrentInfo();
  g_selectedLord->moonring = 1;
  clearHighlighter();
  
  look();
}

void recruitMen(void){
  // code at 64df (JPd from 6503)
  swapWithGarrison(20);
  clearHighlighter();
  think();
}

void guardMen(void){
  swapWithGarrison(-20);
  clearHighlighter();
  think();
}

void battle(void){
  
  clearHighlighter();
  doMove();
}

