/*
* $Log: features.c,v $
* Revision 1.10  2004/01/31 15:21:24  Default
* default declaration warning fix
*
* Revision 1.9  2004/01/31 14:50:04  Default
* buffer copying to screen moved to module blit
*
* Revision 1.8  2004/01/30 22:34:59  Default
* Smooth panning effect
*
* Revision 1.7  2004/01/27 20:19:13  richq
* The temp lord object has changed - it's now a pointer to the actual
* table entry and gets rid of the need for all the copying
* from temp to the table and back again.
*
* Revision 1.6  2004/01/27 19:35:29  richq
* Silly underflow bug...
*
* Revision 1.5  2004/01/25 21:12:07  richq
* Some messing about with clear screen order.
*
* Revision 1.4  2004/01/22 20:07:01  richq
* Moved map out of ROM and out of this module!
*
* Revision 1.3  2004/01/21 18:59:56  Default    
* Optimised a tad by drawing to an ewram buffer.
*
* Revision 1.2  2004/01/19 06:55:21  Default
* WIP. Working, now to optimise a bit.
*
* Revision 1.1.1.1  2004/01/17 09:32:34  Default
* Lords Of Midnight GBA
*
*
*/
#include "screenmode.h"
#include "gba.h"
#include "gfx.h"
#include "features.h"
#include "location.h"
#include "lords.h"
#include "places.h"

#include "blit.h"

// first the look up tables:
typedef struct {
int dy;    // location y adjuster
int dx;    // location x adjuster
int xadj;  // screen x adjuster
int y;     // screen y position
int size;  // scale
} landscape_t;

// Landscaping Location Calculation Table
const landscape_t landscape[] = {
     {5, 4, 86, 64, 7 },        // far away
     {4, 5, 114, 64, 7 },
     {6, 2, 41, 64, 7 },
     {6, 1, 21, 64, 7 },
     {6,-1, -21, 64, 7 },
     {6, 0,  0, 64, 7 },
     {5, 3, 69, 63, 7 },
     {3, 5, 131, 63, 7 },
     {4, 4, 100, 63, 7 },
     {5, 2, 48, 63, 6 },
     {5, 1, 25, 63, 6 },
     {5,-1, -25, 63, 6 },
     {5, 0,  0, 62, 6 },
     {4, 3, 82, 62, 6 },
     {3, 4, 118, 62, 6 },
     {4, 2, 59, 62, 6 },
     {3, 3, 100, 61, 5 },
     {4, 1, 31, 61, 5 },
     {4,-1, -31, 61, 5 },
     {4, 0,  0, 60, 5 },
     {3, 2, 75, 59, 5 },
     {2, 3, 125, 59, 5 },
     {3, 1, 41, 58, 4 },
     {3,-1, -41, 58, 4 },
     {3, 0,  0, 57, 4 },
     {2, 2, 100, 57, 4 },
     {2, 1, 59, 53, 3 },
     {1, 2, 141, 53, 3 },
     {2, 0,  0, 51, 2 },
     {1, 1, 100, 43, 1 },
     {1, 0,  0, 32, 0 },      // close
};


// table from f050. Pointers into 9000
const unsigned short int terrain[][8] = {
  { 0x0000,0x023B,0x03C5,0x04DD,0x05CA,0x0674,0x06DE,0x072F }, // Mountain
  { 0x076E,0x0A82,0x0C76,0x0D93,0x0E6E,0x0F00,0x0F5D,0x0FAB }, // Citadel
  { 0x0FD8,0x120F,0x1384,0x1476,0x153F,0x15D1,0x1632,0x1677 }, // Forest
  { 0x16AA,0x1819,0x18F7,0x1994,0x1A1C,0x1A6E,0x1AA4,0x1AC9 }, // Henge
  { 0x1AE1,0x1C42,0x1D2D,0x1DCC,0x1E4E,0x1EB1,0x1EF8,0x1F2F }, // Tower
  { 0x1F5B,0x204B,0x20E0,0x213E,0x2191,0x21C1,0x21E6,0x21FC }, // Village
  { 0x220B,0x22D1,0x235C,0x23BF,0x2419,0x245D,0x248D,0x24B2 }, // Downs
  { 0x24C7,0x25CE,0x2692,0x26F5,0x2744,0x2773,0x2797,0x27AE }, // Keep
  { 0x27C2,0x2835,0x2884,0x28B7,0x28DD,0x28F5,0x290A,0x291B }, // SnowHall
  { 0x2926,0x295A,0x2981,0x299E,0x29AE,0x29BB,0x29C5,0x29CC }, // lake
  { 0x29D0,0x2BEB,0x2D50,0x2E46,0x2F14,0x2FA8,0x3001,0x303F }, // Wastes
  { 0x3073,0x317F,0x3231,0x32A5,0x330D,0x3359,0x338D,0x33B0 }, // Ruin
  { 0x33CC,0x3567,0x3688,0x3753,0x3807,0x388B,0x38EC,0x3930 }, // lith
  { 0x395C,0x39E0,0x3A3E,0x3A83,0x3ABF,0x3AE8,0x3B09,0x3B24 }, // Cavern
  { 0x3B35,0x3D22,0x3E4A,0x3F11,0x3FB8,0x403E,0x408C,0x40C2 }, // Army
};


int getLocation ( int x, int y ) {
  if (x < 0 || x > 0x3f || y <= 0 || y > 0x3d)
    return 10;
  return g_map[x+(y-1)*64]&0xf;
}

int AdjustMapPos ( int pos, int type, int dx, int dy )
{
   switch ( type ) {
       case 0: pos = pos + dy; break;
       case 1: pos = pos - dy; break;
       case 2: pos = pos + dx; break;
       case 3: pos = pos - dx; break;
   }
   return pos ;
}

void drawLocation(int distance, int xtype, int ytype, int scrtype ) {
  int locx,locy;
  int   x;
  // work out the real y coordinate
  // i.e. y coordinate of this location
  locy = AdjustMapPos (  g_selectedLord->location.y,
                         ytype,
                         landscape[distance].dx,
                         landscape[distance].dy );

  // work out the real x coordinate
  locx = AdjustMapPos (  g_selectedLord->location.x,
                         xtype,
                         landscape[distance].dx,
                         landscape[distance].dy );

  // actually get the location data...
  // use map for this
  
  int feature = getLocation ( locx, locy ) ;

  // work out where on screen, x coordinate this item should be
  x = 320;
  switch ( scrtype ) {
    // case 0 = right side, looking straight
    case 0: x += landscape[distance].xadj ;          break;
    // case 1 = left side, looking straight
    case 1: x -= landscape[distance].xadj ;          break;
    // case 2 = right side, looking diagonal
    case 2: x = (x+landscape[distance].xadj)-100 ;   break;
    // case 3 = left side, looking diagonal
    case 3: x = (x-landscape[distance].xadj)+100 ;   break;
  }

  // and draw it
  drawFeature ( feature,
                landscape[distance].size,
                x,
                landscape[distance].y);
}


// Landscaping Direction Adjustment tables
// used in AdjustMapPos:
// 0: pos = pos + dy; 
// 1: pos = pos - dy; 
// 2: pos = pos + dx; 
// 3: pos = pos - dx; 
// dx, dy come from landscape table at the given distance
const int xAdjustments[] = { 3,2,0,0,2,3,1,1,3 };
const int yAdjustments[] = { 1,1,3,2,0,0,2,3,1 };
void drawPanorama(int dir) {
  
  // clear out the tiles..
  
  clearBuffer();
  
  // looking along a straight line
  int howLftScr = 1;
  int howRhtScr = 0;
  // adjust for diagonal
  if (dir&1 ) {
    howLftScr = 2;
    howRhtScr = 3;
  }
  int ii;
  // step through all the entries in the landscaping table
  for (ii=0; ii<31; ii++ ) {
    // draw location on the left of the screen
    drawLocation( ii,
                  xAdjustments[dir],
                  yAdjustments[dir],
                  howLftScr );

    // draw locations on the right of the screen  
    drawLocation( ii,
                  xAdjustments[dir+1],
                  yAdjustments[dir+1],
                  howRhtScr );
  }
  //wait_vsync_int();
  
  if (g_panning == PAN_RIGHT) {
    // pan right... scroll left
    scrollBufferLeft();
  }
  else if (g_panning == PAN_LEFT) {
    // pan left, scroll right
    scrollBufferRight();
  } else {
    clear_view();
    wait_vsync_int();
    blit_buffer();
    setup_view();
  }
  g_panning = 0;
}





void fillScanLine(int x1, int x2, u8 inkOrPaper, u8 * scanline) {
  int x;
  u8 col = inkOrPaper?1:15;
  for ( x = x1; x <= x2; x++)
    scanline[x] = col;
}


void drawFeature(int feature, int size, int xi, int yi) {
    
  if (feature >= 15 || feature < 0)
    return;
  
  //int y = (192-16 - 68) - yi;    // 176 - y in = bottom of feature.
  // this makes y go from 0:
  // close objects have a y of 32, far away ones y = 64
  // far away objects should be draw further up the screen
  // e.g. for close objects, this would be 77.
  // in the draw loop, this y value decreases - objects are therefore 
  // drawn bottom to top!
  int y = (109 - yi);         // 176 - y in = bottom of feature.
                            // max height is just 78. So could do this with
                            // 10 * 30 = 300 tiles. Or 320 if I 
                            // use the "smooth scrolling" idea.
                            // Chaos used 600 tiles, so this leaves me loads for
                            // everything else.
  int x = xi - 256;
  int index = terrain[feature][size];
  
  int height = featureData[index++];
  int y_offset = 0;
  //u8 scanline[128];
  int startx, endx;
  while ( height-- != 0) {
    y_offset = y*256;
    for (;;) {
      
      int flags = featureData[index++];
      // bit 7 = More than 1 group of drawing operations
      // for this scan line
      // bit 6 = Should we draw in ink or paper
      // bits 0 - 5 = No of drawing instructions to follow
      
      int moreOnLine = (flags & 0x80);
      int inkOrPaper = (flags & 0x40) ? 0 : 1 ;   // 0 = paper, 1 = ink
      int opCount = flags & 0x3f;

      // draw a horizontal line from x1 to x2
      int x1 = featureData[index++];
      int x2 = featureData[index++];
      startx = x1;
      endx = x2;
      drawLineToBuffer(x+x1, x+x2, y_offset, inkOrPaper?1:15);
      
      // draw on the edges in Ink Always
      drawSetToBuffer(x+x1, y_offset, 1);
      drawSetToBuffer(x+x2, y_offset, 1);

      // reverse the last color
      inkOrPaper = (inkOrPaper+1)&1;
      while ( (opCount--) != 0 ) {
        // bit 7 = 1 = Draw A Line from
        // bits 0 - 6 = x coordinate
        flags = featureData[index++];
        x1 = flags & 0x7f ;
        if (flags&0x80) {
          x2 = featureData[index++];
          drawLineToBuffer(x+x1, x+x2, y_offset, inkOrPaper?1:15);

        }
        else {
          drawSetToBuffer(x+x1, y_offset, inkOrPaper?1:15);
        }
      
      }
      if ( !moreOnLine ) break;
    }
    y--;
  }
  
  
}

