/*
* $Log: gfx.c,v $
* Revision 1.8  2004/01/31 15:31:25  Default
* Palettes moved to here. prototype declaration fixes.
* copying from buffer -> vram moved to blit.c
*
* Revision 1.7  2004/01/30 22:42:41  Default
* Code added for the smooth panning/scrolling effect.
*
* Revision 1.6  2004/01/27 19:36:05  richq
* Fading fixed for all layers.
*
* Revision 1.5  2004/01/25 21:16:11  richq
* Got rid of unused code.. plus clear text stuff changed.
*
* Revision 1.4  2004/01/22 22:31:15  richq
* Added the nightime darkness - a straight forward palette swap.
* Moved the text palette init stuff in here too, was in shield.c
*
* Revision 1.3  2004/01/22 20:14:13  richq
* Night palette, more functions added to header, sprite priority variable.
*
* Revision 1.2  2004/01/21 19:05:08  Default
* Big changes and additions. Buffer code added for drawing features.
* Sprite code: initialising, removal and moving for the "object" graphics
* Speccy to GBA graphics conversion code added.
*
* Revision 1.1.1.1  2004/01/17 09:32:34  Default
* Lords Of Midnight GBA
*
*
*/

#include "dma.h"
#include "gba.h"
#include "sprite.h"
#include "screenmode.h"

#include "gfx.h"
#include "stdlib.h"

#include "memory_defs.h"

int g_panningOn = 1;
int g_panning;

// tile data for the sky, ground and fonts starts in char base block 1
// char base block 0 holds the main "features" graphics
#define HORIZON_SKY 1
#define HORIZON_GROUND 2

#define FONT_COL_INDEX 15

#define HORIZON_SKY_PAL (16+1)
#define HORIZON_GROUND_PAL (16+2)

#define BACKGROUND_INK (1)
#define BACKGROUND_PAPER (15)

u8 * const g_buffer = (u8*)ADDR_G_BUFFER;

Bg bg_level;
Bg bg_text;
Bg bg_horizon;

OAMEntry sprites[128];
unsigned char fadedup;
void draw_pixel_paper(unsigned short x, unsigned short y);
void draw_pixel_ink(unsigned short x, unsigned short y);

// gamma correction = sqrt(col*4)
// bright is 223  -> 30  
// non bright is 191 ->
#define BRIGHT 30
#define NONBRT 27
const u16 brightPal[] = {
  RGB16(0,0,0),                //black,
  RGB16(0,0,BRIGHT),           //blue,
  RGB16(BRIGHT,0,0),           //red,
  RGB16(BRIGHT,0,BRIGHT),      // purple
  RGB16(0,BRIGHT,0),           //green,
  RGB16(0,BRIGHT,BRIGHT),      // cyan 
  RGB16(BRIGHT,BRIGHT,0),      // yellow
  RGB16(BRIGHT,BRIGHT,BRIGHT), // white
};
const u16 nonBrightPal[] = {
  RGB16(0,0,0),                 //black,
  RGB16(0,0,NONBRT),            //blue,
  RGB16(NONBRT,0,0),            //red,
  RGB16(NONBRT,0,NONBRT),       // purple
  RGB16(0,NONBRT,0),            //green,
  RGB16(0,NONBRT,NONBRT),       // cyan 
  RGB16(NONBRT,NONBRT,0),       // yellow
  RGB16(NONBRT,NONBRT,NONBRT),  // white
};

// clears out the video memory, which may have been set already by the boot menu
void clear_vram(void) {
  u16 i;
  u32 zero = 0;
  // fade down
  REG_BLDMOD = BLDMOD_FIRST_BG1 | BLDMOD_FIRST_BG0 | BLDMOD_FIRST_BG2 |
               BLDMOD_FIRST_BG3 |
               BLDMOD_FIRST_OBJ | BLDMOD_SECOND_BD | BLDMOD_MODE(3);
  REG_COLEY = 16;

  REG_DMA3SAD = (u32)&zero;
  REG_DMA3DAD = (u32)&(VideoBuffer[0]); // VRAM
  REG_DMA3CNT = 0x3100 |DMA_SOURCE_FIXED|DMA_16NOW; // 16 bit transfers from gfx array via DMA

  for (i = 0; i < 256; i++)
    BGPaletteMem[i] = 0;



}

void clear_view(void) {
  
  bg_level.x_scroll = 8;
  int x,y;
  // sets the view field
  for (x = 0; x < 32; x++) {
    for (y = 0; y < 10; y++) {
      bg_level.mapData[x+y*32] = 0;
    }
  }
  
  // clears a square in the arena
//  u16 zero = 0;
//  u16 startOAM = 16;
//  u16 square_count = 16*10;
//  
//  REG_DMA3SAD = (u32)&zero; // 16 bit reads into VRAM
//  REG_DMA3DAD = (u32)&(bg_level.tileData[startOAM]); // 16 bit reads into VRAM
//  REG_DMA3CNT = (32*square_count) |DMA_SOURCE_FIXED|DMA_16NOW; // 16 bit transfers from gfx array via DMA    
}

void clear_text_from(int startY) {
  int x,y;
  for (x = 0; x < 32; x++) {
    for (y = startY; y < 20; y++) {
      bg_text.mapData[x+y*32] = 0;
    }
  }
}

void clear_text(void) {
  clear_text_from(0);
}



void setup_view(void) {
  int x, y;
  
  int i = 1;
  for (y=0; y < 10; y++) {
    for (x=0; x < 32; x++) {
      bg_level.mapData[x+y*32] = i;
      i++;
    }
  }
  
  bg_level.x_scroll = 8;
  bg_level.y_scroll = 256-42;
  
  UpdateBackground(&bg_level);
}

void init_backgrounds(void) {

  // create a bg  
  bg_level.number = 0;				//background number 0-2
  bg_level.charBaseBlock = 0;
  bg_level.screenBaseBlock = 28;
  bg_level.colorMode = BG_COLOR_16;
  bg_level.size = TEXTBG_SIZE_256x256; 
  bg_level.mosiac = 0;
  bg_level.x_scroll = 8;
  bg_level.y_scroll = 256-42;
  EnableBackground(&bg_level);
  REG_BG0CNT &= 0xFFFD;
  REG_BG0CNT |= 1;
  UpdateBackground(&bg_level);
  
  
  bg_text.number = 1;				//background number 0-2
  bg_text.charBaseBlock = 1;
  bg_text.screenBaseBlock = 29;
  bg_text.colorMode = BG_COLOR_16;
  bg_text.size = TEXTBG_SIZE_256x256; 
  bg_text.mosiac = 0;
  bg_text.x_scroll = 0;
  bg_text.y_scroll = 0;
  EnableBackground(&bg_text);
  REG_BG1CNT &= 0xFFFD;
  REG_BG1CNT |= 0;
  UpdateBackground(&bg_text);
  int x, y;
  for (y=0; y < 20; y++) {
    for (x=0; x < 30; x++) {
      bg_text.mapData[x+y*32] = 0;
    }
  }
  
  
  bg_horizon.number = 2;				//background number 0-2
  bg_horizon.charBaseBlock = 1;
  bg_horizon.screenBaseBlock = 30; // 31 is the largest base block for the map data
  bg_horizon.colorMode = BG_COLOR_16;
  bg_horizon.size = TEXTBG_SIZE_256x256; 
  bg_horizon.mosiac = 0;
  bg_horizon.x_scroll = 0;
  bg_horizon.y_scroll = 0;
  EnableBackground(&bg_horizon);
  REG_BG2CNT &= 0xFFFD;
  REG_BG2CNT |= 2;
  UpdateBackground(&bg_horizon);
  
  
  
}

void set_think_palette(void) {
  BGPaletteMem[BACKGROUND_INK]   = RGB16(0,0,0);
  BGPaletteMem[BACKGROUND_PAPER] = RGB16(NONBRT,NONBRT,NONBRT);
  BGPaletteMem[HORIZON_SKY_PAL] = RGB16(NONBRT,NONBRT,NONBRT);
  BGPaletteMem[HORIZON_GROUND_PAL] = RGB16(NONBRT,NONBRT,NONBRT);
}


void set_night_palette(void) {
  BGPaletteMem[HORIZON_SKY_PAL]   = RGB16(0,0,0);
  BGPaletteMem[HORIZON_GROUND_PAL] = RGB16(0,0,0);
}

void set_horizon_palette(void) {
  BGPaletteMem[BACKGROUND_INK] = RGB16(0,0,BRIGHT);
  BGPaletteMem[BACKGROUND_PAPER] = RGB16(BRIGHT,BRIGHT,BRIGHT);

  BGPaletteMem[HORIZON_SKY_PAL] = RGB16(0,0,BRIGHT);
  BGPaletteMem[HORIZON_GROUND_PAL] = RGB16(BRIGHT,BRIGHT,BRIGHT);
}

void set_palette_time(int time) {
  
  if (time == 0) {
    wait_vsync_int();
    BGPaletteMem[BACKGROUND_INK] =   RGB16(0,0,NONBRT);
    BGPaletteMem[BACKGROUND_PAPER] = 0x0;
    
    BGPaletteMem[HORIZON_SKY_PAL]   = RGB16(0,0,NONBRT);
    BGPaletteMem[HORIZON_GROUND_PAL] = RGB16(0,0,0);
  }
  else {
    set_horizon_palette();  
  }
  
}
 

void set_horizon(void) {
  set_horizon_palette();
  int x, y;
  for (y=0; y < 11; y++) {
    for (x=0; x < 32; x++) {
      bg_horizon.mapData[x+y*32] = HORIZON_SKY|(1<<12);
    }
  }
  for (y= 0; y < 9; y++) {
    for (x=0; x < 32; x++) {
      bg_horizon.mapData[352+x+y*32] = HORIZON_GROUND|(1<<12);
    }
  }
  int i = 0;
  int offset = HORIZON_SKY*16;
  for (i = 0; i < 32; i++)
    bg_horizon.tileData[offset+i] = 0x1111;
  
  offset = HORIZON_GROUND*16;
  for (i = 0; i < 32; i++)
    bg_horizon.tileData[offset+i] = 0x2222;
  
}


const unsigned short paperMasks[] = {
  0x000F,
  0x00F0,
  0x0F00,
  0xF000
};

void fade_up(void) {
  u8 loop;
  
  REG_BLDMOD = BLDMOD_FIRST_BG1 | BLDMOD_FIRST_BG0 | BLDMOD_FIRST_BG2 |
               BLDMOD_FIRST_BG3 |
               BLDMOD_FIRST_OBJ | BLDMOD_SECOND_BD | BLDMOD_MODE(3);
  
  for (loop = 0; loop < 17; loop+=2) {
    wait_vsync_int();
    REG_COLEY = (16-loop);
  }
  wait_vsync_int();
  REG_COLEY = 0;
  fadedup = 1;
}     

void fade_down(void) {
  u8 loop;
  
  REG_BLDMOD = BLDMOD_FIRST_BG1 | BLDMOD_FIRST_BG0 | BLDMOD_FIRST_BG2 |
               BLDMOD_FIRST_BG3 |
               BLDMOD_FIRST_OBJ | BLDMOD_SECOND_BD | BLDMOD_MODE(3);
  
  for (loop = 0; loop < 17; loop+=2) {
    wait_vsync_int();
    REG_COLEY = loop;
  }
  wait_vsync_int();
  REG_COLEY = 16;
  fadedup = 0;
}


// 39478 in sna, da1b in memory
#define fontGFX _binary_font_bin_start 
extern u8 fontGFX[];
// load the font data into tile ram
void load_font(void) {
  // the font is stored in 1 bit per pixel format
  // expand it to 4bpp
  // each letter is 8*8, 4bpp = 8*4 pixels = 32 u8s per char
  
  int i, j, k, m;
  
  // amount to shift by
  int shift; 
  // four pixels, 4 bits each
  u16 pixelQuad; 
  // start index in tile ram
  u32 address = FONT_TILE*16;
  // offset in fontGfX array of the character
  int offset = 0;
  u8 theByte;
  u8 bitMask;
  for (j = 0; j < 91; j++) {
    //offset = j*8;
    // loop over characters
    for (i = 0; i < 8; i++) {
      // loop over rows
      theByte = fontGFX[offset++];
      if (theByte != 0) {
        // has data
        // 66
        for (k = 0; k < 2; k++) {
          // each row requires 2 writes to vram
          
          pixelQuad = (u16)0x0000;
          // create the u16 of 4 pixels:
          for (m = 0; m < 4; m++) {
            bitMask = 1;
            // pixel k*4+m on this row
            shift = k*4 + m;
            
            if (shift != 7)
              bitMask <<= (7-shift);
            
            if ( (theByte & bitMask) != 0) {
//              if (m == 0)
//                pixelQuad = FONT_COL_INDEX;
//              else
              pixelQuad |= paperMasks[m];//(FONT_COL_INDEX<<(m*4));
            }
          }
          bg_text.tileData[address++] = pixelQuad;
          
        }
      } else {
        // the byte is empty, draw a blank row
        for (m = 0; m < 2; m++) {
          bg_text.tileData[address++] = 0x0000; // pixel quad of 00 00
        }
      }
    }
  }  
}

// print character at row, col in text bg
// use palette pal
void print_text(char text, int row, int col, int pal) {
  if (text != 0x0) {
    bg_text.mapData[row+col*32] = (FONT_TILE + (text)-0x20)|((pal&0xf)<<12);
  }
  
}

void CopyOAM(void)
{
  unsigned short int loop;
  unsigned short int* temp;
  temp = (unsigned short int*)sprites;
  for(loop = 0; loop < 128*4; loop++)
  {
  	OAM[loop] = temp[loop];
  }

}

void MoveSprite(OAMEntry* sp, signed short int x, signed short int y)
{
    if(x < 0)			//if it is off the left corect
		x = 512 + x;
	if(y < 0)			//if off the top corect
		y = 256 + y;

    /* because x is 9 bits, we need to clear out bits 10-15,
	 so we dont mess up any rotation data */
    x &= 0x01FF;
	sp->attribute1 = sp->attribute1 & 0xFE00;  //clear the old x value
	sp->attribute1 = sp->attribute1 | x;

	sp->attribute0 = sp->attribute0 & 0xFF00;  //clear the old y value
	sp->attribute0 = sp->attribute0 | y;

}


void move_sprite(int id, int x, int y) {
  if (id < 128) {
    MoveSprite(&sprites[id], x, y);
    CopyOAM();
  }
}

void remove_sprite(int id) {
  sprites[id].attribute0 = SIZE_DOUBLE;
}


/*All sprites are 64*64, col256 so can "hard code" OAM entry calculations*/
u32 get_oam64(int id) {
  return id * 64*64/2;
}

void init_sprite64(int id, int priority) {
  sprites[id].attribute0 = SQUARE|COLOR_256;
  sprites[id].attribute1 = SIZE_64;
  sprites[id].attribute2 = PRIORITY(priority) | (get_oam64(id)/16);
  // OAM = 16 * sprite entry
}

const u8 * gfxTable;
void createTile(u8 * pixels, int tile, int pen, int paper) {
  
  const u8 * data = &gfxTable[tile*8];
  // read the image from offset...
  // the image is made up of 1 bit per pixel graphics
  // so for an image of 16*16, you would need 16 rows of 2 bytes
  
  int width = 1;
  int x, y, n;
  for (y = 0; y < 8; y++) {
    for (x = 0; x < width; x++) {
      // read in the byte of gfx data...
      // now expand to 8 pixels
      for (n = 0; n < 8; n++) {
        int pixelIndex = (x*8 + n) + y*8; /* *width; */
        if (n != 7)
          pixels[pixelIndex] = *data & (1<<(7-n));
        else
          pixels[pixelIndex] = *data & 1;
          
        if (pixels[pixelIndex] == 0) 
          pixels[pixelIndex] = paper;
        else
          pixels[pixelIndex] = pen;
      }
      data++;
    }
  }
  
}


// copy tile pixels into vram for this sprite at the given row/columb
// for the given sprite - all sprites are 64*64 so this is allowed
void drawImage(u8 * pixels, u8 col, u8 row, u8 sprite) { 
  // to work out tile, use the old x+y*width formula
  // width is 8, byt the way
  int i =  col + row*8;// tile count;
  int nOAM = get_oam64(sprite);
  REG_DMA3SAD = (u32)pixels; // 8 bit reads into gfx array
  REG_DMA3DAD = (u32)&OAMData[nOAM+i*32]; // 16 bit reads into VRAM
  REG_DMA3CNT = 32 |DMA_16NOW; // 16 bit transfers from gfx array via DMA
  
}

// assume sprite is 64*64!!
void clearSpriteRAM(u8 sprite) {
  u32 zero = 0;
  int nOAM = get_oam64(sprite);
  REG_DMA3SAD = (u32)&zero; // 8 bit reads into gfx array
  REG_DMA3DAD = (u32)&OAMData[nOAM]; // 16 bit reads into VRAM
  REG_DMA3CNT = 32*4*8 |DMA_SOURCE_FIXED|DMA_32NOW; // 16 bit transfers from gfx array via DMA
  
}

void init_gfxbuffer(void) {
  
  // init text colours too...
  
//  BGPaletteMem[16*2+0xf] = RGB16(26,0,0); // red for text
//  BGPaletteMem[16*3+0xf] = RGB16(30,0,30);  // purple for night
//  BGPaletteMem[16*4+0xf] = RGB16(0,27,0);  // green *thinks*
//  BGPaletteMem[16*5+0xf] = RGB16(0,31,31); //cyan
//  BGPaletteMem[16*6+0xf] = RGB16(31,31,0); //yellow
  
  BGPaletteMem[16*2+0xf] = RGB16(NONBRT,0,0); // red for text
  BGPaletteMem[16*3+0xf] = RGB16(BRIGHT,0,BRIGHT);  // purple for night
  BGPaletteMem[16*4+0xf] = RGB16(0,NONBRT,0);  // green *thinks*
  BGPaletteMem[16*5+0xf] = RGB16(0,BRIGHT,BRIGHT); //cyan
  BGPaletteMem[16*6+0xf] = RGB16(BRIGHT,BRIGHT,0); //yellow
  
}

void clearBuffer(void) {
  u32 zero = 0;
  REG_DMA3SAD = (u32)&zero; // 8 bit reads into gfx array
  REG_DMA3DAD = (u32)g_buffer; // 16 bit reads into VRAM
  REG_DMA3CNT = (256*80/4) |DMA_SOURCE_FIXED|DMA_32NOW; // 16 bit transfers from gfx array via DMA
}


void drawLineToBuffer(int x1, int x2, int yOffset, int colour) {
  if ( x1 < 0 ) x1 = 0;
  if ( x2 > 255) x2 = 255;
  if (x1 > x2)
    return;
  int i;
  
  for (i = x1; i <= x2; i++)
    g_buffer[yOffset +i] = colour;
}

void drawSetToBuffer(int x, int yOffset, int colour) {
  if ( x<0 ) return;
  if ( x>255) return;
  g_buffer[yOffset+x] = colour;
}


// direction 
// 0 = right
// 1 = left
void scrollBuffer(int direction) {
  int i, j, scroller;
  u8 * buffer;
  u16 * pAddress;
  u16 val;
  int thisColumn;
  
  // "Carry" is set if x_scroll/8 leaves a half or not - i.e. if 
  // the buffer is currently misaligned w.r.t tile mem 
  int carry = bg_level.x_scroll&4;
  
  // When carry NOT set copy one less columns on next scroll
  // 18 columns are repeated, we don't need to copy them
  // i.e. if cary set, copy an extra column 
  int max_val = (carry)?14:13;
  // When carry NOT set, need to start source at buffer+4 
  // This is because the buffer is 4 pixels out compared to current panorama
  int bufferOffset = (carry)?0:4;
  
  // so for !carry, max_val = 12 in loop... 
  // x offset = 144 + bufferOffset + scroller*8
  // x offset = 144 + 4 + 12*8 = 244 (seems right - goes to 252, leaving 4 stray pixels)
  // for carry, max_val = 13 in loop... 
  // x offset = 144 + bufferOffset + scroller*8
  // x offset = 144 + 0 + 13*8 = 248 (seems right - goes to 256, the full buffer width)
  
  for (scroller = 0; scroller < max_val; scroller++) {
    // get the column to start the copying at...
    if (direction) {
      thisColumn = (bg_level.x_scroll/8+30);
    }
    else {
      thisColumn = (bg_level.x_scroll/8+0xff);
    }
    
    if (carry && scroller == max_val-1) {
      // last column copy is just past the scroll
      if (direction) {
        thisColumn++;
      } else {
        thisColumn--;
      }
    }
    
    thisColumn &= 0x1f;
    // maybe +31? this would get us back to col 0... which may be what we want
    // possibly for right(), would be -1 instead of +30
    
    // destination in vram for this column - +16 as the first tile is blank on purpose
    pAddress = &bg_level.tileData[16+ thisColumn*16];
    // source tile data for this column in ram
    // 144 represents the 18 columns of repeated data.
    if (direction) {
      buffer = &g_buffer[144 + bufferOffset+scroller*8];
    } else {
      buffer = &g_buffer[96 + bufferOffset-scroller*8];
    }
    
    // 10 tiles in each row, copy them:
    for (j = 0; j < 10; j++) {
      
      // this copies 1 tile:
      for (i = 0; i < 8; i++) {
        
        // first 4 pixels on this row:
        val  = (*buffer++);
        val |= (*buffer++)<<4;
        val |= (*buffer++)<<8;
        val |= (*buffer++)<<12;
        
        *pAddress++ = val;
        
        // next 4 pixels on this row:
        val  = (*buffer++);
        val |= (*buffer++)<<4;
        val |= (*buffer++)<<8;
        val |= (*buffer++)<<12;
        
        *pAddress++ = val;
        
        // increase buffer 1 row:
        buffer+=248;
      }
      // increase tile vram pointer a row of tiles:
      pAddress += 31*16;
    }
    // only scroll 12 times - 12*8 = 96 pixels
    // the occasional 13th column copy is to cover misalignment 
    // and may not even be needed?
    if (scroller < 12) {
      wait_vsync_int();
      // I certainly hope this is expandable to scroll right by just doing "-8"
      if (direction)
        bg_level.x_scroll += 8;
      else
        bg_level.x_scroll -= 8;
        
      bg_level.x_scroll &= 0xff;
      UpdateBackground(&bg_level);
    }
  }
  // final shuffle left half a tile: Total scroll is 100 pixels
  // this 4 pixel mis-alignment is what causes all the problems
  if (direction)
    bg_level.x_scroll += 4;
  else
    bg_level.x_scroll -= 4;
    
  bg_level.x_scroll &= 0xff;
  UpdateBackground(&bg_level);
  
}

void scrollBufferLeft(void) {
  scrollBuffer(1);
}

void scrollBufferRight(void) {
  scrollBuffer(0);
}


