/*
* $Log: lords.c,v $
* Revision 1.7  2004/01/30 22:48:50  Default
* Dynamic memory stuff removed - replaced by static ewram assignment
*
* Revision 1.6  2004/01/27 20:19:13  richq
* The temp lord object has changed - it's now a pointer to the actual
* table entry and gets rid of the need for all the copying
* from temp to the table and back again.
*
* Revision 1.5  2004/01/25 21:24:48  richq
* Update stats at night, recalculate graphics when horse lost/found
*
* Revision 1.4  2004/01/22 22:33:32  richq
* Icefear uses just the lower 8 bits in the courage calculation...
*
* Revision 1.3  2004/01/22 20:23:40  richq
* Most of the specific controls stuff removed and added to think.c, look.c etc.
*
* Revision 1.2  2004/01/22 20:03:52  richq
* The Think Screen code is added.
*
* Revision 1.1  2004/01/21 19:20:04  Default
* Stuff to do with lords. Looking, thinking, moving, etc.
*
*/

#include "qmath.h"
#include "gba.h"
#include "lords.h"
#include "stdlib.h"
#include "string.h"
#include "savearea.h"
#include "icefear.h"
#include "army.h"

#include "memory_defs.h"

#define lordData _binary_lorddata_bin_start
extern u8 lordData [];

int g_lordScreen = 0;
int g_gameover = 0;
lord_t * const g_lordTable = (lord_t *)ADDR_G_LORDTABLE;
// the currently selected lord
lord_t * g_selectedLord;
u8 g_selectedLordID;


void copyRomLord(lord_t * lordTo, u8 * lordFrom) {
  lordTo->location.x = *lordFrom++;
  lordTo->location.y = *lordFrom++;
  lordTo->location.direction = *lordFrom++;
  lordTo->time = *lordFrom++;
  lordTo->firstname = *lordFrom++;
  lordTo->title = *lordFrom++;
  lordTo->moonring = *lordFrom++;
  lordTo->graphic = *lordFrom++;
  lordTo->riders = *lordFrom++;
  lordTo->riders_energy = *lordFrom++;
  lordTo->warriors = *lordFrom++;
  lordTo->warriors_energy = *lordFrom++;
  lordTo->battle_area = *lordFrom++;
  lordTo->riders_lost = *lordFrom++;
  lordTo->warriors_lost = *lordFrom++;
  lordTo->killed = *lordFrom++;
  lordTo->riders_killed = *lordFrom++;
  lordTo->warriors_killed = *lordFrom++;
  lordTo->battle_status = *lordFrom++;
  lordTo->alive = *lordFrom++;
  lordTo->energy = *lordFrom++;
  lordTo->strength = *lordFrom++;
  lordTo->cowardess = *lordFrom++;
  lordTo->recruiting_key = *lordFrom++;
  lordTo->recruited_by = *lordFrom++;
  lordTo->courage = *lordFrom++;
  lordTo->unused = *lordFrom++;
  lordTo->hidden = *lordFrom++;
  lordTo->race = *lordFrom++;
  lordTo->horse = *lordFrom++;
  lordTo->object = *lordFrom++;
  lordTo->deathby = *lordFrom++;
}



void setSelectedLord(int id) {
  g_selectedLordID = id;
  g_selectedLord = &g_lordTable[id];
}


/*
updateCurrentInfo : 
makes the temp g_selectedLord object point to the entry in the table.
Calculates which location we are looking at.
Fetches all details for the current location (terrain, armies, other lords here etc).

based on efc2
*/
void updateCurrentInfo(void) {
  
  setSelectedLord(g_saveArea.selectedLord);
  lookingAt(&g_selectedLord->location);
  workOutLocationDetails(&g_selectedLord->location);
  setSelectedLord(g_saveArea.selectedLord);
  
}


void init_lordtable(void) {
  
  int i;
  for (i = 0; i < 32; i++) {
    copyRomLord(&g_lordTable[i], &lordData[i*32]);
  }
}


/*
calculates the courage for the lord 
based on 65dc
*/
void calculateCourage(void) {
  // gba div
  signed long icefear_div_7, modulus;
  FastIntDivide(icefear&0xff, 7, &icefear_div_7, &modulus);
  
  int crg = g_selectedLord->cowardess - icefear_div_7;
  
  if (crg > 0) {
    crg /= 8;
  }
  g_selectedLord->courage = crg<8?crg:7;
  //saveSelectedLord();  
}

// fec0
// remember to reset the selected lord when you're done!
int getRecruitableID(void) {
  if (g_lordsHere < 2) {
    return 0;
  }
  int i;
  int newAlly = 0;
  int recruiting_key = g_selectedLord->recruiting_key;
  for (i = 0; i < g_lordsHere; i++) {
    // retrieve selected lord
    setSelectedLord(g_charTable[i]);
    if (g_selectedLord->moonring != 1) {
      // is not already an ally, can we recruit him?
      if (g_selectedLord->recruited_by & recruiting_key) {
        // yes!
        newAlly = g_charTable[i];
        // no need to keep looking, there are never 
        // more than 1 new ally in a location
        break;
      }
    }
  }
  return newAlly;
}

// 6736
void updateStats(int increment) {
  
  int newval = g_selectedLord->energy + increment + 9;
  g_selectedLord->energy = (newval > 0x7f)?0x7f:newval;
  
  newval = g_selectedLord->riders_energy + increment + 6;
  g_selectedLord->riders_energy = (newval > 0x7f)?0x7f:newval;
  
  newval = g_selectedLord->warriors_energy + increment + 4;
  g_selectedLord->warriors_energy = (newval > 0x7f)?0x7f:newval;
  
  
}


void resetLord(int lord) {
  setSelectedLord(lord);
  
  // update the stats by B
  updateStats(g_selectedLord->time/2);
  // call 6736
  g_selectedLord->time = 16;
  if (g_selectedLord->alive) {
    g_selectedLord->battle_status = 0xff;
    
    g_selectedLord->riders_lost = 0x0;
    g_selectedLord->warriors_lost = 0x0;
    g_selectedLord->killed = 0x0;
    g_selectedLord->riders_killed = 0x0;
    g_selectedLord->warriors_killed = 0x0;
    
  }
  
}

// update the lords for the dawn 
void resetLordStats(void) {
  int i;
  for (i = 0; i < 32; i++) {
    resetLord(i);
  }
}

const u8 lordGFX[] =
 {0x08, 0x01, 
  0x10, 0x0f, 
  0x0e, 0x00, 
  0x0a, 0x03, 
  0x0b, 0x04, 
  0x0c, 0x0c, 
  0x06, 0x06 };


// calculate the new graphics
// call this when we gain/lose a horse or wahtever
void calculateGFX(void) {
  
  g_selectedLord->graphic = lordGFX[(g_selectedLord->race-1)*2 + g_selectedLord->horse];
}

// lose/find moonring
void toggleControl(int toggleFlag) {
  int i;
  for ( i = 2; i < 32; i++) {
    setSelectedLord(i);
    if (g_selectedLord->moonring == toggleFlag) {
      g_selectedLord->moonring ^= 2;
      //saveSelectedLord();
    }
  }
  
}
