/*
* $Log: tokens.h,v $
* Revision 1.4  2004/01/26 20:41:49  richq
* Added a way to append the area name to the token buffer.
* Plus some tidying up and bios div instead of %.
*
* Revision 1.3  2004/01/22 20:32:49  richq
* Bug fix for the MildlyCold code in .c, .h has a function name corrected
* and the current graphics row made "public".
*
* Revision 1.2  2004/01/22 19:53:35  richq
* Helper functions added (addUppercaseToken, etc).
* Number to words added.
* Removed the need to reset the token list manually, now done when
* the list is flushed.
*
* Revision 1.1  2004/01/21 19:33:09  Default
* Token parsing - tokens are stored using 5 bit bytes compressed together,
* the format is [length][length * char] - code here expands the tokens,
* prints them out and other tokeny things.
*
*/

/*
* Everything to do with text tokens and token parsing
*/

#ifndef _TOKENS_H
#define _TOKENS_H

#include "gba.h"
#include "location.h"

extern int g_currentY;    // current row
extern int g_capsLock;
extern int g_isPlural;

void init_token_buffers(void);

// print the tokens setting all the data for start position, indent, etc
void print_tokens(int y, int x, int indent, int maxx, int palette);
// print tokens using current settings, but change the palette only
void print_tokens_pal(int palette);

void createHeStandsString(location_t * position);
// creates the lord's title
void createLordTitle(int title, int name);
// as above, but capitalised first letter (if needed)
void createLordTitleUppercase1(int title, int name);

/*Takes an int value and converts it to words 
  adds the tokens for the words to the token list */
void numberToWords(int value);
void addToTokenList(const u8 * array);
void addSingleToken(u8 t);
void addPluralisedToken(u8 t);
void addNameToken(int name);
void addUppercaseToken(void);
void addNewlineToken(void);
void addLiteralToken(char lit);
void addDomainTokens(locdat_t * loc);
void addDirection(int direction);
void addNewlineToken(void);

void addAreaToken(int area);
/* 
add a descriptive string for the state of something
quantity should be 0-7
positive is the positive descriptive token (e.g. bold)
neegative is the negative descriptive token (e.g. tired)
*/
void addMildlyCold(int quantity, int positive, int negative);

// reset heap back to start
// not needed if we always reset heap after printing, 
// which is pretty much the way it was done.
//void reset_heap(void);

#endif

